/**************************************************************************/
/*                                                                        */
/*  Thibaut Balabonski, Sylvain Conchon, Jean-Christophe Filliâtre,       */
/*  Kim Nguyen, Laurent Sartre                                            */
/*                                                                        */
/*  Informatique - MP2I/MPI - CPGE 1re et 2e années.                      */
/*  Cours et exercices corrigés. Éditions Ellipses, 2022.                 */
/*                                                                        */
/*  https://www.informatique-mpi.fr/                                      */
/*                                                                        */
/**************************************************************************/

// Arbres

#include "prelude.h"

#include "tree.h"

tree *tree_create(int v) {
  tree *t = malloc(sizeof(struct Tree));
  t->value = v;
  t->children = NULL;
  t->next = NULL;
  return t;
}

void tree_add_first_child(tree *t, tree *c) {
  assert(c->next == NULL);
  c->next = t->children;
  t->children = c;
}

void tree_add_last_child(tree *t, tree *c) {
  if (t->children == NULL) {
    t->children = c;
    return;
  }
  tree *p = t->children;
  tree *n = p->next;
  while (n != NULL) {
    p = n;
    n = n->next;
  }
  p->next = c;
}

int tree_size(tree *t) {
  int s = 1;
  for (tree *c = t->children; c != NULL; c = c->next) {
    s += tree_size(c);
  }
  return s;
}

void tree_preorder(tree *t) {
  putchar('(');
  putchar(t->value);
  for (tree *c = t->children; c != NULL; c = c->next) {
    tree_preorder(c);
  }
  putchar(')');
}

// attention : suppose qu'il n'y a pas de partage de sous-arbre
void tree_delete(tree *t) {
  tree *c = t->children;
  while (c != NULL) {
    tree *p = c->next;
    tree_delete(c);
    c = p;
  }
  free(t);
}

